/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package main;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import main.parser.KNNResultFileParser;
import messif.objects.LocalAbstractObject;
import messif.objects.util.AbstractObjectList;
import messif.objects.util.RankedAbstractObject;
import messif.objects.util.StreamGenericAbstractObjectIterator;
import messif.operations.RankingSingleQueryOperation;

/**
 * Class for handling IO operations
 *
 * @author Vladimir Mic, xmic@fi.muni.cz
 */
public class IOTools {

    /**
     *
     * @param className name of the class of serialised objects. Extends the
     * LocalAbstractObject class
     * @param filePath path to the text file with serialised ojects
     * @return iterator over LocalAbstractObjects in the file
     */
    public static StreamGenericAbstractObjectIterator<LocalAbstractObject> getIterator(String className, String filePath) {
        try {
            return getIterator(Class.forName(className), filePath);
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(IOTools.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     *
     * @param clazz class of serialised objects. Extends the LocalAbstractObject
     * class
     * @param filePath path to the text file with serialised ojects
     * @return iterator over LocalAbstractObjects in the file
     */
    public static StreamGenericAbstractObjectIterator getIterator(Class clazz, String filePath) {
        try {
            File file = new File(filePath);
            if (!file.exists()) {
                return null;
            }
            return new StreamGenericAbstractObjectIterator(clazz, filePath);
        } catch (IOException ex) {
            Logger.getLogger(IOTools.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     *
     * @param clazz class of serialised objects. Extends the LocalAbstractObject
     * class
     * @param filePath path to the text file with serialised ojects
     * @return list with LocalAbstractObjects in the file
     */
    public static AbstractObjectList<LocalAbstractObject> getObjects(Class clazz, String filePath) {
        return getObjects(clazz, filePath, -1);
    }

    /**
     *
     * @param clazz class of serialised objects. Extends the LocalAbstractObject
     * class
     * @param filePath path to the text file with serialised ojects
     * @param count maximum number of the returned objects
     * @return list with LocalAbstractObjects in the file
     */
    public static AbstractObjectList getObjects(Class clazz, String filePath, int count) {
        StreamGenericAbstractObjectIterator<LocalAbstractObject> it = getIterator(clazz, filePath);
        return new AbstractObjectList(it, count);
    }

    /**
     * Returns map of the radii for query objects.
     *
     * @param k defines the k-th nearest neighbour for which the range is
     * returned
     * @param groundTruthPath text file with the evaluated queries
     * @return map<String, Float> where String is the locatorURI of the query
     * object (LocatorURI of the LocalAbstractObject), and float is the range to
     * its k-th nearest neighbour according to the text file at the
     * groundTruthPath
     */
    public static Map<String, Float> parseRadiiForKNN(int k, String groundTruthPath) {
        List<KNNResultFileParser.EvaluatedQuery> groundTruth = new KNNResultFileParser(groundTruthPath).getEvaluatedQueries();
        Map<String, Float> ret = new HashMap<>();
        for (KNNResultFileParser.EvaluatedQuery evaluatedQuery : groundTruth) {
            String locatorURI = evaluatedQuery.getLocatorURI();
            Float dist = evaluatedQuery.getDistances().get(k - 1);
            ret.put(locatorURI, dist);
        }
        return ret;
    }

    /**
     *
     * @param list list of strings that are/contain numbers
     * @return list of integers takes from the strings
     */
    public static List<Integer> transformStringsToIntegers(List<String> list) {
        List<Integer> ret = new ArrayList<>();
        if (list == null || list.isEmpty()) {
            return ret;
        }
        for (int i = 0; i < list.size(); i++) {
            String s = list.get(i);
            while (!Character.isDigit(s.charAt(0))) {
                s = s.substring(1);
            }
            ret.add(Integer.parseInt(s));
        }
        return ret;
    }

    /**
     * Print answer of the similarity search query to the printstream
     *
     * @param op similarity search operation from the messif library (e.g.
     * KnnQueryOperation, RangeQueryOperation)
     * @param os print stream where to print
     */
    public static void printAnswer(RankingSingleQueryOperation op, PrintStream os) {
        Iterator<RankedAbstractObject> result = op.getAnswer();
        os.println("INFO: Algorithm processed: " + op.toString());
        int counter = 0;
        try {
            while (result.hasNext()) {
                RankedAbstractObject res = result.next();
                if (counter > 0) {
                    os.write(", ".getBytes());
                }
                String s = res.getDistance() + ": " + res.getObject().getLocatorURI();
                os.write(s.getBytes());

                counter++;
            }
            os.write('\n');
        } catch (IOException ex) {
            Logger.getLogger(IOTools.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

}
